<?php

/*
CREATE TABLE users (
  user_id int(11) NOT NULL auto_increment,
  username varchar(13) NOT NULL default '',
  password varchar(30) NOT NULL default '',
  email varchar(100) NOT NULL default '',
  real_name varchar(30) NOT NULL default '',
  date_joined datetime NOT NULL default '0000-00-00 00:00:00',
  admin enum('0','1') NOT NULL default '0',
  PRIMARY KEY  (user_id),
  UNIQUE KEY user_id_2 (user_id),
  KEY user_id (user_id)
) TYPE=MyISAM;
*/

require_once("./include/cdatabase.inc.php");

class cUser
{
    // $IS_LOGGED_IN set to TRUE if user is logged in
    var $IS_LOGGED_IN;

    // variables to store user information
    var $user_id;
    var $username;
    var $email;
    var $real_name;
    var $g_id;
    var $date_joined;
    var $admin;

    // class variables
    var $error_message;
    var $debug;

    // min and max lengths for username and password defined in config.inc.php
    var $min_password_length;
    var $max_password_length;
    var $min_username_length;
    var $max_username_length;


    // constructor
    function cUser()
    {
        $this->IS_LOGGED_IN = FALSE;
        $this->debug = FALSE;
        $this->min_password_length = 5;
        $this->max_password_length = 13;
        $this->min_username_length = 4;
        $this->max_username_length = 13;
    }


    function authenticate($username, $password)
    {
        $db = new cDatabase;

        if ($this->verify_password($username, $password)) {

                $result = $db->query("SELECT * FROM users WHERE username = '$username'");
                @extract($db->fetch_array($result), EXTR_PREFIX_ALL, "db");

                $this->user_id      = $db_user_id;
                $this->username     = $db_username;
                $this->email        = $db_email;
                $this->real_name    = $db_real_name;
                $this->g_id         = $db_g_id;
                $this->date_joined  = $db_date_joined;
                $this->admin        = $db_admin;
                $this->IS_LOGGED_IN = TRUE;

                if ($this->debug) {
                    echo "User id: $user_id<br>";
                    echo "Username: $username<br>";
                    echo "Email: $email<br>";
                    echo "real name: $real_name<br>";
                    echo "Admin: $admin<br>";
                }

                return TRUE;
             } else {
                return FALSE;
             }

    }

    function change_password($username, $old_password, $password1, $password2)
    {

        $db = new cDatabase;

        if ($this->verify_password($username, $old_password)) {

            if ($password1 != $password2) {
                $this->error("Your passwords do not match.");
                return FALSE;
            }

            if ((strlen($password1) < $this->min_password_length) || (strlen($password1) > $this->max_password_length)) {
                $this->error("Your password must be more than " . $this->min_username_length . " characters and less than "
                . $this->max_password_length . ".");
                return FALSE;
            }

            $encrypted_password = crypt($password1);
            $result = $db->query("UPDATE users SET password = '$encrypted_password' WHERE username = '$username'");

            if ($this->debug) {
                echo "old password: $old_password<br>";
                echo "password1: $password1<br>";
                echo "password2: $password2<br>";
                echo "username: $username<br>";
                echo "encrypted password: $encrypted_password<br>";
            }

            return TRUE;

        } else {
            return FALSE;
        }
    }

    function error($message)
    {
        $this->error_message = $message;
        return TRUE;
    }

    function logout()
    {
        $this->IS_LOGGED_IN = FALSE;
        $this->username = '';
        $this->email = '';
        $this->real_name = '';
        $this->admin = FALSE;
        $this->g_id = '';
        $this->date_joined = '';

        return TRUE;
    }

    // mails lost password to user
    function mail_password($username, $email)
    {
        $db = new cDatabase;
        $result = $db->query("SELECT email FROM users WHERE username = '$username' AND email = '$email'");
        if (!$db->num_rows($result)) {
            $this->error("The username or email address is incorrect.");
            return FALSE;
        } else {
            $password = $this->make_password();
            list($email) = $db->fetch_array($result);
            $subject = "Your lost password.";
            $message = "Your password that you requested had to be reset.  Your new password is:\n" .
                       "$password";
            $header = "From: Password reset <noreply@site.com>\r\n";
            if (!mail($email, $subject, $message, $header)) {
                $this->error("Failed to send email.");
                return FALSE;
            } else {
                $encrypted_password = crypt($password);
                $result = $db->query("UPDATE users SET password = '$encrypted_password' WHERE username = '$username'");
                return TRUE;
            }
        }
    }

    // makes a random password
    function make_password($length = 7)
    {
        // thanks to benjones@superutility.net for this code

        mt_srand((double) microtime() * 1000000);

        for ($i=0; $i < $length; $i++) {
            $which = rand(1, 3);
            // character will be a digit 2-9
            if ( $which == 1 ) $password .= mt_rand(0,10);
            // character will be a lowercase letter
            elseif ( $which == 2 ) $password .= chr(mt_rand(65, 90));
            // character will be an uppercase letter
            elseif ( $which == 3 ) $password .= chr(mt_rand(97, 122));
        }
        return $password;
    }

    // registers a new user
    function register($username, $password1, $password2, $email, $real_name, $g_id, $admin)
    {
        $db = new cDatabase;

        if ($this->debug) echo "Username: $username";
        if ((strlen($username) < $this->min_username_length) || (strlen($username) > $this->max_username_length)) {
            $this->error("Your username must be more than " . $this->min_username_length . " characters and less than "
                         . $this->max_username_length . ".");
            return FALSE;
        }

        if ($password1 != $password2) {
            $this->error("Your passwords do not match.");
            return FALSE;
        }

        if ((strlen($password1) < $this->min_password_length) || (strlen($password1) > $this->max_password_length)) {
            $this->error("Your password must be more than " . $this->min_username_length . " characters and less than "
                         . $this->max_password_length . ".");
            return FALSE;
        }

        $result = $db->query("SELECT * FROM users WHERE username = '$username'");
        if ($db->num_rows($result)) {
            $this->error("The username you have chosen is already in use.  Please choose another one.");
            return FALSE;
        }

        // http://www.perldoc.com/perl5.6/pod/func/crypt.html for info on the encryption method

        $encrypted_password = crypt($password1);
        $date = $db->format_date();
        $result = $db->query("INSERT INTO users (username, password, email, real_name, g_id, date_joined, admin)
                              VALUES ('$username', '$encrypted_password', '$email', '$real_name', $g_id,
                                      '$date', $admin)");
        return TRUE;
    }

    // updates user account info
    function update($user_id, $username, $email, $real_name, $g_id, $admin)
    {
        $db = new cDatabase;

        if ($this->debug) echo "Username: $username";
        if ((strlen($username) < $this->min_username_length) || (strlen($username) > $this->max_username_length)) {
            $this->error("Your username must be more than " . $this->min_username_length . " characters and less than "
                         . $this->max_username_length . ".");
            return FALSE;
        }

        $result = $db->query("UPDATE users SET username  = '$username',
                                               email     = '$email',
                                               real_name = '$real_name',
                                               g_id      = $g_id,
                                               admin     = $admin
                                               WHERE user_id = $user_id");
        return TRUE;
    }

    function verify_password($username, $password)
    {
        $db = new cDatabase;
        $result = $db->query("SELECT password FROM users WHERE username = '$username'");
        if (!$db->num_rows($result)) {
            $this->error("Username or password is incorrect ...");
            return FALSE;
        }

        list($stored_password) = $db->fetch_array($result);

//        if (!$verified) {
//            $this->error("Your account has not been approved by the administrator yet.");
//            return FALSE;
//        }

        $real_password = crypt($password, $stored_password);

        if ($this->debug) {
            echo "Plain text password passed to function: $password<br>";
            echo "password stored in database: $stored_password<br>";
            echo "real password: $real_password<br>";
        }

        $result = $db->query("SELECT * FROM users WHERE username = '$username' AND password = '$real_password'");
        if ($db->num_rows($result)) {
            return TRUE;
        } else {
            $this->error("Username or password incorrect ...");
            return FALSE;
        }
    }
}